"use strict";
/* eslint-disable @typescript-eslint/no-unsafe-member-access */
/*
 * Copyright 2020-2023 SynTest contributors
 *
 * This file is part of SynTest Framework - SynTest Framework.
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */
Object.defineProperty(exports, "__esModule", { value: true });
exports.OptionGroups = exports.WebsocketEventListenerPlugin = void 0;
const logging_1 = require("@syntest/logging");
const module_1 = require("@syntest/module");
const WebSocket = require("ws");
const handler_1 = require("./handlers/handler");
const AbstractSyntaxTreeModel_1 = require("./models/AbstractSyntaxTreeModel");
const ControlFlowGraphModel_1 = require("./models/ControlFlowGraphModel");
const DependencyModel_1 = require("./models/DependencyModel");
const SearchProgressModel_1 = require("./models/SearchProgressModel");
const SourceModel_1 = require("./models/SourceModel");
const TargetModel_1 = require("./models/TargetModel");
/**
 * This plugin publishes all of the SynTest's events to the specified url with WebSocket protocol.
 *
 * @author Yehor Kozyr
 */
class WebsocketEventListenerPlugin extends module_1.EventListenerPlugin {
    constructor() {
        super("websocket", 
        // eslint-disable-next-line @typescript-eslint/no-unsafe-argument, @typescript-eslint/no-var-requires, unicorn/prefer-module
        require("../../package.json").description);
        WebsocketEventListenerPlugin.LOGGER = (0, logging_1.getLogger)("WebsocketEventListenerPlugin");
    }
    get fid() {
        return this._fid;
    }
    set fid(_fid) {
        this._fid = _fid;
    }
    async connect() {
        const url = this.args.wsUrl;
        if (url === undefined) {
            WebsocketEventListenerPlugin.LOGGER.warn(`There was no websocket url provided to the websocket plugin. Skipping connection.`);
            return;
        }
        this.client = new WebSocket(url);
        const client = this.client;
        await new Promise((resolve, reject) => {
            client.on("error", (error) => {
                WebsocketEventListenerPlugin.LOGGER.error(`Error connecting to server with url: ${url}\n error: ${error.name}, ${error.message}`);
                reject();
            });
            client.on("open", () => {
                WebsocketEventListenerPlugin.LOGGER.info(`Connected to server with url: ${url}`);
                client.send(Buffer.from(JSON.stringify({ event: "publisherPluginStarted", fid: this._fid })));
                resolve();
            });
            client.on("ping", () => {
                client.send("pong");
            });
            client.on("close", () => {
                WebsocketEventListenerPlugin.LOGGER.info(`Disconnected from server with url: ${url}`);
                client.terminate();
            });
        });
    }
    disconnect() {
        if (this.client === undefined) {
            return;
        }
        WebsocketEventListenerPlugin.LOGGER.info(`Terminating client`);
        this.client.terminate();
    }
    async setupEventListener() {
        await this.connect();
        if (this.client === undefined) {
            return;
        }
        process.on("initializeStart", () => (0, handler_1.handler)(this.client, this._fid, "initializeStart", {}));
        process.on("initializeComplete", () => (0, handler_1.handler)(this.client, this._fid, "initializeComplete", {}));
        process.on("preprocessStart", () => (0, handler_1.handler)(this.client, this._fid, "preprocessStart", {}));
        process.on("preprocessComplete", () => (0, handler_1.handler)(this.client, this._fid, "preprocessComplete", {}));
        process.on("processStart", () => (0, handler_1.handler)(this.client, this._fid, "processStart", {}));
        process.on("processComplete", () => (0, handler_1.handler)(this.client, this._fid, "processComplete", {}));
        process.on("postprocessStart", () => (0, handler_1.handler)(this.client, this._fid, "postprocessStart", {}));
        process.on("postprocessComplete", () => (0, handler_1.handler)(this.client, this._fid, "postprocessComplete", {}));
        process.on("instrumentationStart", () => (0, handler_1.handler)(this.client, this._fid, "instrumentationStart", {}));
        process.on("instrumentationComplete", () => (0, handler_1.handler)(this.client, this._fid, "instrumentationComplete", {}));
        process.on("targetRunStart", () => (0, handler_1.handler)(this.client, this._fid, "targetRunStart", {}));
        process.on("targetRunComplete", () => (0, handler_1.handler)(this.client, this._fid, "targetRunComplete", {}));
        process.on("reportStart", () => (0, handler_1.handler)(this.client, this._fid, "reportStart", {}));
        process.on("reportComplete", () => (0, handler_1.handler)(this.client, this._fid, "reportComplete", {}));
        // search events
        process.on("searchInitializationStart", () => (0, handler_1.handler)(this.client, this._fid, "searchInitializationStart", {}));
        process.on("searchInitializationComplete", () => (0, handler_1.handler)(this.client, this._fid, "searchInitializationComplete", {}));
        process.on("searchStart", (searchAlgorithm, subject, budgetManager, _terminationManager) => (0, handler_1.handler)(this.client, this._fid, "searchStart", (0, SearchProgressModel_1.searchProgressModelFormatter)(searchAlgorithm, subject, budgetManager)));
        process.on("searchComplete", 
        // eslint-disable-next-line @typescript-eslint/unbound-method
        (searchAlgorithm, subject, budgetManager, _terminationManager) => (0, handler_1.handler)(this.client, this._fid, "searchComplete", (0, SearchProgressModel_1.searchProgressModelFormatter)(searchAlgorithm, subject, budgetManager)));
        process.on("searchIterationStart", (searchAlgorithm, subject, budgetManager, _terminationManager) => (0, handler_1.handler)(this.client, this._fid, "searchIterationStart", (0, SearchProgressModel_1.searchProgressModelFormatter)(searchAlgorithm, subject, budgetManager)));
        process.on("searchIterationComplete", (searchAlgorithm, subject, budgetManager, _terminationManager) => (0, handler_1.handler)(this.client, this._fid, "searchIterationComplete", (0, SearchProgressModel_1.searchProgressModelFormatter)(searchAlgorithm, subject, budgetManager)));
        // analysis events
        process.on("sourceResolvingStart", (rootContext, filePath) => (0, handler_1.handler)(this.client, this._fid, "sourceResolvingStart", (0, SourceModel_1.sourceModelFormatter)(rootContext, filePath)));
        process.on("sourceResolvingComplete", (rootContext, filePath, source) => {
            (0, handler_1.handler)(this.client, this._fid, "sourceResolvingComplete", (0, SourceModel_1.sourceModelFormatter)(rootContext, filePath, source));
        });
        process.on("abstractSyntaxTreeResolvingStart", (rootContext, filePath) => (0, handler_1.handler)(this.client, this._fid, "abstractSyntaxTreeResolvingStart", (0, AbstractSyntaxTreeModel_1.abstractSyntaxTreeModelFormatter)(rootContext, filePath)));
        process.on("abstractSyntaxTreeResolvingComplete", (rootContext, filePath, abstractSyntaxTree) => (0, handler_1.handler)(this.client, this._fid, "abstractSyntaxTreeResolvingComplete", (0, AbstractSyntaxTreeModel_1.abstractSyntaxTreeModelFormatter)(rootContext, filePath, abstractSyntaxTree)));
        process.on("controlFlowGraphResolvingStart", (rootContext, filePath) => (0, handler_1.handler)(this.client, this._fid, "controlFlowGraphResolvingStart", (0, ControlFlowGraphModel_1.controlFlowGraphModelFormatter)(rootContext, filePath)));
        process.on("controlFlowGraphResolvingComplete", (rootContext, filePath, cfp) => (0, handler_1.handler)(this.client, this._fid, "controlFlowGraphResolvingComplete", (0, ControlFlowGraphModel_1.controlFlowGraphModelFormatter)(rootContext, filePath, cfp)));
        process.on("targetExtractionStart", (rootContext, filePath) => (0, handler_1.handler)(this.client, this._fid, "targetExtractionStart", (0, TargetModel_1.targetModelFormatter)(rootContext, filePath)));
        process.on("targetExtractionComplete", (rootContext, filePath, target) => (0, handler_1.handler)(this.client, this._fid, "targetExtractionComplete", (0, TargetModel_1.targetModelFormatter)(rootContext, filePath, target)));
        process.on("dependencyResolvingStart", (rootContext, filePath) => (0, handler_1.handler)(this.client, this._fid, "dependencyResolvingStart", (0, DependencyModel_1.dependencyModelFormatter)(rootContext, filePath)));
        process.on("dependencyResolvingComplete", (rootContext, filePath, dependencies) => (0, handler_1.handler)(this.client, this._fid, "dependencyResolvingComplete", (0, DependencyModel_1.dependencyModelFormatter)(rootContext, filePath, dependencies)));
    }
    getOptions(tool, labels, command) {
        const optionsMap = new Map();
        if (!labels.includes("testing")) {
            return optionsMap;
        }
        if (command !== "test") {
            return optionsMap;
        }
        optionsMap.set("ws-url", {
            alias: [],
            default: undefined,
            description: "The url of the listening WebSocket",
            group: OptionGroups.PublisherWSOptions,
            hidden: false,
            normalize: true,
            type: "string",
        });
        return optionsMap;
    }
    getOptionChoices() {
        return [];
    }
}
exports.WebsocketEventListenerPlugin = WebsocketEventListenerPlugin;
var OptionGroups;
(function (OptionGroups) {
    OptionGroups["PublisherWSOptions"] = "Publishing WS Options:";
})(OptionGroups = exports.OptionGroups || (exports.OptionGroups = {}));
//# sourceMappingURL=WebsocketEventListenerPlugin.js.map